package de.hdm_stuttgart.mi.sd1.interest;

/**
 * Dealing with account balances and interest calculations.
 *
 */
public class Account {

  private static double
      assetInterestRate = 1.5,  // applied to positive balances / assets
      debtInterestRate = 15.;   // applied to negative balances / debt

  private static int accountLimit = 10000;

  private double balance;       // Balances are defined on a per-account basis. Thus "static" must not appear here.

  /**
   * Create a new account having a balance of 0.
   */
  public Account() {            // Default Constructor. This one has no parameters creating
    this(0);             // an initially empty account.
  }

  /**
   * Create a new account of given initial balance.
   *
   * @param balance The account's initial balance.
   */
  public Account(double balance) {  // Non-default constructor creating an account
    setBalance(balance);            // of given balance.
  }

  /**
   * @return The account's current balance.
   */
  public double getBalance() {
    return balance;
  }
  /**
   * Setting a (possibly) new balance.
   *
   * @param balance The desired new balance value. If greater than 10000
   *      a warning message is being issued and the accounts current value will be retained.
   */
  public void setBalance(double balance) {
    if (balance <= accountLimit) {
      this.balance = balance; // "this" required to resolve name shadowing conflict.
    } else {
      System.out.println("Balance" + balance + " exceeds " + accountLimit);
    }
  }
  /**
   * Setting the interest rate common to all accounts. This one
   * will be applied to positive balances only. For handling
   * negative balances see {{@link #setDebtInterestRate(double)}.
   *
   * @param assetInterestRate The desired (global) interest rate.
   */
  public static void setAssetInterestRate(double assetInterestRate) {
    Account.assetInterestRate = assetInterestRate;
  }
  /**
   * @return The current global interest rate
   */
  public static double getAssetInterestRate() {
    return assetInterestRate;
  }

  /**
   * @return The debt interest rate value.
   */
  public static double getDebtInterestRate() {
    return debtInterestRate;
  }

  /**
   * This interest rate will be applied to negative balances. In contrast
   * {{@link #setAssetInterestRate(double)} will handle positive balance values.
   *
   * @param debtInterestRate The debt interest rate's value.
   */
  public static void setDebtInterestRate(double debtInterestRate) {
    Account.debtInterestRate = debtInterestRate;
  }

  /**
   * <p>Convenience method to {{@link #applyInterest(int)}}: Adding one year's interest to the current balance according
   * to:</p>
   *
   * \[ balance_{1} = balance (1 + {interestRate\over 100}) \]
   *
   */
  public void applyInterest() {
    applyInterest(1);
  }

  /**
   * <p>Adding the accumulated interest of a given number of years to the current balance according to:</p>
   *
   * \[ balance_{years} = balance\ {\left(1 + {interestRate\over 100}\right)}^{years} \]
   *
   *<p>Positive balances will receive the value of {@link #setAssetInterestRate(double)} whereas
   * calculation of negative  balances will be based on {{@link #setDebtInterestRate(double)}.</p>
   *
   * @param years the given time period.
   */
  public void applyInterest(int years) {

    final double annualInterestFactor;
    if (0 < balance) {                                      // Asset type balance
      annualInterestFactor = 1 + assetInterestRate / 100;
    } else {                                                // Debt type balance
      annualInterestFactor = 1 + debtInterestRate / 100;
    }
    for (int i = 0; i < years; i++) {
      balance *= annualInterestFactor;
    }
  }

  /**
   * Calculate the expected annual interest without changing the account's balance.
   *
   * @return Annual interest to be expected
   */
  public double getYearlyInterest() {
    return balance * assetInterestRate / 100;
  }
}